# -*- coding: UTF-8 -*-
"""
Name: argparser.py
Porpose: Check for command line arguments before starting
Compatibility: Python3
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: September.11.2020
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
from shutil import which
import argparse
from videomass3.vdms_sys.msg_info import current_release


def args():
    """
    Parser of the users inputs (positional/optional arguments)

    USE:
        videomass -h
    """
    parser = argparse.ArgumentParser(description=('GUI for FFmpeg and '
                                                  'youtube-dl'),)
    parser.add_argument('-v', '--version',
                        help="show the current version and exit",
                        action="store_true",
                        )
    parser.add_argument('-c', '--check',
                        help=('List of executables used by Videomass '
                              'found on the system'),
                        action="store_true",
                        )

    argmts = parser.parse_args()

    if argmts.check:
        listing = ['ffmpeg',
                   'ffprobe',
                   'ffplay',
                   'youtube-dl',
                   'atomicparsley'
                   ]
        print('List of executables used by Videomass:')
        for required in listing:
            if required == 'atomicparsley':
                opt = '[Optional]'
            else:
                opt = '[Required]'
            if which(required, mode=os.F_OK | os.X_OK, path=None):
                print("\t%s '%s' ..Ok" % (opt, required))
            else:
                print("\t%s '%s' ..Not Installed" % (opt, required))

    elif argmts.version:
        crel = current_release()
        print('%s version %s released on %s' % (crel[0], crel[2], crel[3]))

    else:
        print("Type 'videomass -h' for help.")



# -*- coding: UTF-8 -*-
"""
Name: playlist_indexing.py
Porpose: shows a dialog box for setting playlist indexing
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Aug.22.2021 *-pycodestyle- compatible*
########################################################

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx
import wx.lib.mixins.listctrl as listmix


class ListCtrl(wx.ListCtrl,
               listmix.ListCtrlAutoWidthMixin,
               listmix.TextEditMixin):
    """
    A listctrl with the ability to be editable.
    """

    def __init__(self, parent, ID, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=0):
        wx.ListCtrl.__init__(self, parent, ID, pos, size, style)

        listmix.ListCtrlAutoWidthMixin.__init__(self)
        self.populate()
        listmix.TextEditMixin.__init__(self)

    def populate(self):
        """populate with default colums"""
        self.InsertColumn(0, '#', width=30)
        self.InsertColumn(1, _('URL'), width=400)
        self.InsertColumn(2, _('Playlist Items'), width=200)


class Indexing(wx.Dialog):
    """
    Shows a dialog box for setting playlist indexing.

    """
    get = wx.GetApp()  # get data from bootstrap
    OS = get.appset['ostype']
    appdata = get.appset

    if get.appset['icontheme'] in ('Breeze-Blues',
                                   'Videomass-Colours'):
        BACKGROUND = '#11303eff'
        FOREGROUND = '#959595'
        YELLOW = '#dfb72f'

    elif get.appset['icontheme'] in ('Breeze-Blues',
                                     'Breeze-Dark',
                                     'Videomass-Dark'):
        BACKGROUND = '#1c2027ff'
        FOREGROUND = '#87ceebff'
        YELLOW = '#dfb72f'

    else:
        BACKGROUND = '#e6e6faff'
        FOREGROUND = '#191970ff'
        YELLOW = '#988313'

    GREEN = '#008000'
    RED = '#ff0000ff'
    HELPME = _('Click on "Playlist Items" column to specify indices of '
               'the videos in the playlist separated by commas like: '
               '"1,2,5,8" if you want to download videos indexed 1, 2, '
               '5, 8 in the playlist.\n\n'
               'You can specify range: "1-3,7,10-13" it will download the '
               'videos at index 1, 2, 3, 7, 10, 11, 12 and 13.\n'
               )

    def __init__(self, parent, url, data):
        """
        NOTE constructor:: with 'None' not depend from videomass.
        With 'parent, -1' if close videomass also close mediainfo window
        """
        self.urls = url
        self.data = data

        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)

        # ------ Add widget controls
        self.lctrl = ListCtrl(self,
                              wx.ID_ANY,
                              style=wx.LC_REPORT |
                              wx.SUNKEN_BORDER |
                              wx.LC_SINGLE_SEL |
                              wx.LC_HRULES |
                              wx.LC_VRULES
                              )
        self.tctrl = wx.TextCtrl(self,
                                 wx.ID_ANY, "",
                                 style=wx.TE_MULTILINE |
                                 wx.TE_READONLY |
                                 wx.TE_RICH2
                                 )

        # ------ Properties
        self.SetTitle(_('Playlist video items to download'))
        self.SetMinSize((800, 400))
        self.lctrl.SetMinSize((800, 200))
        self.tctrl.SetMinSize((800, 200))

        if Indexing.OS == 'Darwin':
            self.lctrl.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            self.tctrl.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.BOLD))
        else:
            self.lctrl.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            self.tctrl.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))

        self.tctrl.SetBackgroundColour(Indexing.BACKGROUND)

        # ------ set Layout
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_1.Add(self.lctrl, 0, wx.ALL | wx.EXPAND, 5)
        grid_sizer_1 = wx.GridSizer(1, 1, 0, 0)
        sizer_1.Add(grid_sizer_1, 1, wx.EXPAND, 0)
        grid_sizer_1.Add(self.tctrl, 0, wx.ALL | wx.EXPAND, 5)

        # ------ bottom layout for buttons
        grid_btn = wx.GridSizer(1, 2, 0, 0)
        gridexit = wx.BoxSizer(wx.HORIZONTAL)
        btn_reset = wx.Button(self, wx.ID_CLEAR, _("Reset"))
        grid_btn.Add(btn_reset, 0, wx.ALL, 5)
        btn_close = wx.Button(self, wx.ID_CANCEL, "")
        gridexit.Add(btn_close, 0, wx.ALL, 5)
        self.btn_ok = wx.Button(self, wx.ID_OK, _("Apply"))
        gridexit.Add(self.btn_ok, 0, wx.ALL, 5)
        grid_btn.Add(gridexit, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, 0)

        # ------ final settings:
        sizer_1.Add(grid_btn, 0, wx.EXPAND)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        self.Layout()

        # ------ set listctrl data:
        index = 0
        for link in url:
            self.lctrl.InsertItem(index, str(index + 1))
            self.lctrl.SetItem(index, 1, link)
            if not self.data == {'': ''}:
                for key, val in self.data.items():
                    if key == link:
                        self.lctrl.SetItem(index, 2, val)
            index += 1

        # ----------------------Binding (EVT)----------------------#
        self.lctrl.Bind(wx.EVT_LIST_BEGIN_LABEL_EDIT, self.on_edit_begin)
        self.lctrl.Bind(wx.EVT_LIST_END_LABEL_EDIT, self.on_edit_end)
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_close)
        self.Bind(wx.EVT_BUTTON, self.on_ok, self.btn_ok)
        self.Bind(wx.EVT_BUTTON, self.on_reset, btn_reset)

        self.textstyle()

    def textstyle(self):
        """
        clear log messages and set text style on textctrl box
        """

        self.tctrl.Clear()
        self.tctrl.SetDefaultStyle(wx.TextAttr(Indexing.FOREGROUND))
        self.tctrl.AppendText('%s' % Indexing.HELPME)
    # ------------------------------------------------------------------#

    def getvalue(self):
        """
        This method return values via the interface getvalue()
        """
        diz = {}
        index = 0
        for rows in self.urls:
            if self.lctrl.GetItem(index, 2).GetText():
                diz[rows] = self.lctrl.GetItem(index, 2).GetText()
            index += 1
        return diz

    # ----------------------Event handler (callback)----------------------#

    def on_edit_end(self, event):
        """
        A check is made on the strings entered
        """
        wxd = wx.DateTime.Now()
        date = wxd.Format('%H:%M:%S')

        errdigit = _('ERROR: Enter only digits, commas and hyphens. If you '
                     'have multiple items, use commas to separate them. Use '
                     'the hyphen for the range. Spaces are not allowed.')
        errdigit2 = _('ERROR: You have to start and end with a digit.')
        errhyph = _('ERROR: Make sure the hyphen is between two digits.')
        errhyph3 = _('ERROR: A range must include only two digits and a '
                     'hyphen, for example "20-25" .')
        errcomma = _('ERROR: Make sure you use commas only to separate '
                     'indices and / or ranges, for example "1,3,5,10-15"')
        errgen = _('ERROR: The entered string contains redundant characters.')
        errchan = _('WARNING: The URL contains a channel. Unable to determine '
                    'playlist indexing.')

        row_id = event.GetIndex()  # Get the current row
        # col_id = event.GetColumn()  # Get the current column
        new_data = event.GetLabel()  # Get the changed data
        # cols = self.lctrl.GetColumnCount()  # Get the total number of col
        # rows = self.lctrl.GetItemCount()  # Get the total number of rows

        if not new_data == '':

            last = len(new_data) - 1
            csplit = new_data.split(',')  # split by commas
            hsplit = new_data.split('-')  # split by hyphens
            allow = ['-', ',', '0', '1', '2', '3',
                     '4', '5', '6', '7', '8', '9']

            if 'channel' in self.lctrl.GetItem(row_id, 1).GetText():
                self.tctrl.SetDefaultStyle(wx.TextAttr(Indexing.YELLOW))
                self.tctrl.AppendText('\n%s: %s' % (date, errchan))
                event.Veto()  # contain channel
            elif [char for char in new_data if char not in allow]:
                self.tctrl.SetDefaultStyle(wx.TextAttr(Indexing.RED))
                self.tctrl.AppendText('\n%s: %s' % (date, errdigit))
                event.Veto()  # only enter digits, commas and hyphens
            elif not new_data[0].isdigit() or not new_data[last].isdigit():
                self.tctrl.SetDefaultStyle(wx.TextAttr(Indexing.RED))
                self.tctrl.AppendText('\n%s: %s' % (date, errdigit2))
                event.Veto()  # must start with digit
            elif [hyph for hyph in csplit if hyph.startswith('-')]:
                self.tctrl.SetDefaultStyle(wx.TextAttr(Indexing.RED))
                self.tctrl.AppendText('\n%s: %s' % (date, errhyph))
                event.Veto()  # malformed
            elif [hyph for hyph in csplit if hyph.endswith('-')]:
                self.tctrl.SetDefaultStyle(wx.TextAttr(Indexing.RED))
                self.tctrl.AppendText('\n%s: %s' % (date, errhyph))
                event.Veto()  # malformed
            elif [comma for comma in hsplit if comma.startswith(',')]:
                self.tctrl.SetDefaultStyle(wx.TextAttr(Indexing.RED))
                self.tctrl.AppendText('\n%s: %s' % (date, errcomma))
                event.Veto()  # malformed
            elif '' in csplit or '' in hsplit:
                self.tctrl.SetDefaultStyle(wx.TextAttr(Indexing.RED))
                self.tctrl.AppendText('\n%s: %s' % (date, errgen))
                event.Veto()  # malformed
            elif [rng.split('-') for rng in csplit if '-' in rng
                  if len(rng.split('-')) > 2]:
                self.tctrl.SetDefaultStyle(wx.TextAttr(Indexing.RED))
                self.tctrl.AppendText('\n%s: %s' % (date, errhyph3))
                event.Veto()  # range must include two digits and one hyphen
            else:
                self.tctrl.SetDefaultStyle(wx.TextAttr(Indexing.GREEN))
                self.tctrl.AppendText('\n%s: Assigned: %s ' % (date, new_data))
    # ------------------------------------------------------------------#

    def on_edit_begin(self, event):
        """
        Columns 0 and 1 must not be editable.
        """

        if event.GetColumn() in (0, 1):
            event.Veto()
        else:
            event.Skip()  # or event.Allow()
    # ------------------------------------------------------------------#

    def on_reset(self, event):
        """
        Reset all items on editable columns and clear log messages
        """
        rows = self.lctrl.GetItemCount()  # Get the total number of rows
        for row in range(rows):
            self.lctrl.SetItem(row, 2, '')

        self.textstyle()
    # ------------------------------------------------------------------#

    def on_close(self, event):
        """
        Close the dialog
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def on_ok(self, event):
        """
        if you enable self.Destroy(), it delete from memory all data
        event and no return correctly. It has the right behavior if
        not used here, because it is called in the main frame.

        Event.Skip(), work correctly here. Sometimes needs to disable
        it for needs to maintain the view of the window (for exemple).
        """
        self.getvalue()
        # self.Destroy()
        event.Skip()



# -*- coding: UTF-8 -*-
"""
File Name: make_filelog.py
Porpose: log file generator
Compatibility: Python3, Python2
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: May.16.2021
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""

import time
import os


def write_log(logfile, logdir):
    """
    Before starting a process, a log file is created from this
    template and then written later by the process.
    see also `vdms_sys/ctrl_run.py` .

    - logfile,  log name from which the command was generated
    - logdir, log files location
    """
    if not os.path.isdir(logdir):
        try:
            os.makedirs(logdir, mode=0o777)
        except OSError as error:
            return error

    current_date = time.strftime("%c")  # date/time
    path = os.path.join(logdir, logfile)

    with open(path, "a", encoding='utf8') as log:
        log.write("""
[DATE]:
%s

[LOCATION]:
%s

[VIDEOMASS]:
""" % (current_date, path))

    return None



# -*- coding: UTF-8 -*-
"""
Name: YtdlExecDL.py
Porpose: long processing task with youtube-dl executable
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Aug.13.2021
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
from threading import Thread
import time
import itertools
import subprocess
import platform
import wx
from pubsub import pub
if not platform.system() == 'Windows':
    import shlex


def logwrite(cmd, sterr, logname, logdir):
    """
    writes youtube-dl commands and status error during
    threads below
    """
    if sterr:
        apnd = "...%s\n\n" % (sterr)
    else:
        apnd = "%s\n\n" % (cmd)

    with open(os.path.join(logdir, logname), "a", encoding='utf8') as log:
        log.write(apnd)


class YtdlExecDL(Thread):
    """
    YtdlExecDL represents a separate thread for running
    youtube-dl executable with subprocess class to download
    media and capture its stdout/stderr output in real time .

    """
    get = wx.GetApp()  # get videomass wx.App attribute
    LOGDIR = get.appset['logdir']
    FFMPEG_URL = get.appset['ffmpeg_bin']
    EXECYDL = get.appset['EXECYDL']
    APPTYPE = get.appset['app']

    if get.appset['playlistsubfolder'] == 'true':
        SUBDIR = '%(uploader)s/%(playlist_title)s/%(playlist_index)s - '
    else:
        SUBDIR = ''

    if not platform.system() == 'Windows':
        LINE_MSG = _('Unrecognized error')
    else:
        if os.path.isfile(EXECYDL):
            LINE_MSG = ('\nERROR: MSVCR100.dll is missing!\nPlease, install: '
                        '"Microsoft Visual C++ 2010 Redistributable Package '
                        '(x86)"\nwhich can be found at:\n'
                        'https://download.microsoft.com/download/1/6/5/165255'
                        'E7-1014-4D0A-B094-B6A430A6BFFC/vcredist_x86.exe\n')
        else:
            LINE_MSG = _('Unrecognized error')
    # -----------------------------------------------------------------------#

    def __init__(self, varargs, logname):
        """
        Attributes defined here:
        self.stop_work_thread:  process terminate value
        self.args['urls']:          urls list
        self.args['opt']:           option strings to adding
        self.args['outtmpl']:       options template to renaming on pathname
        self.args['code']:          Format Code, else empty string ''
        self.args['outputdir']:     pathname destination
        self.count:         increases with the progressive loops
        self.countmax:      length of self.args['urls']
        self.logname:       title log name for logging

        """
        self.args = {'urls': varargs[1],
                     'opt': varargs[4][0],
                     'outtmpl': varargs[4][1],
                     'nooverwrites': varargs[4][2],
                     'restrictfn': varargs[4][3],
                     'pl_items': varargs[4][4],
                     'code': varargs[6],
                     'outputdir': varargs[3]
                     }
        self.stop_work_thread = False  # process terminate
        self.count = 0
        self.countmax = len(varargs[1])
        self.logname = logname

        if (platform.system() == 'Windows' or
                YtdlExecDL.APPTYPE == 'appimage'):
            self.ssl = '--no-check-certificate'

        else:
            self.ssl = ''

        Thread.__init__(self)
        self.start()  # start the thread (va in self.run())

    def run(self):
        """
        Subprocess initialize thread.

        """
        for url, code in itertools.zip_longest(self.args['urls'],
                                               self.args['code'],
                                               fillvalue='',
                                               ):
            if 'playlist' in url or '--yes-playlist' in self.args['opt']:
                outtmpl = YtdlExecDL.SUBDIR + self.args['outtmpl']
            else:
                outtmpl = self.args['outtmpl']

            if self.args['pl_items'].get(url):
                pl_items = ('--playlist-items %s ' %
                            self.args['pl_items'].get(url))
            else:
                pl_items = ''

            format_code = '--format %s' % (code) if code else ''
            cmd = ('"{0}" {1} --newline --ignore-errors {8} -o '
                   '"{2}/{3}" {4} {5} {10} --ignore-config {9} "{6}" '
                   '--ffmpeg-location "{7}"'.format(YtdlExecDL.EXECYDL,
                                                    self.ssl,
                                                    self.args['outputdir'],
                                                    outtmpl,
                                                    format_code,
                                                    self.args['opt'],
                                                    url,
                                                    YtdlExecDL.FFMPEG_URL,
                                                    self.args['nooverwrites'],
                                                    self.args['restrictfn'],
                                                    pl_items,
                                                    ))
            self.count += 1
            count = 'URL %s/%s' % (self.count, self.countmax)
            dest = 'Destination:  "%s"' % self.args['outputdir']
            com = ('%s\nSource: %s\n%s\n\n'
                   '[COMMAND]\n%s' % (count, url, dest, cmd))

            wx.CallAfter(pub.sendMessage,
                         "COUNT_EVT",
                         count=count,
                         fsource='Source: %s' % url,
                         destination=dest,
                         duration=100,
                         end='',
                         )
            logwrite(com,
                     '',
                     self.logname,
                     YtdlExecDL.LOGDIR,
                     )  # write n/n + command only

            if not platform.system() == 'Windows':
                cmd = shlex.split(cmd)
                info = None
            else:
                # Hide subprocess window on MS Windows
                info = subprocess.STARTUPINFO()
                info.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            try:
                with subprocess.Popen(cmd,
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.STDOUT,
                                      bufsize=1,
                                      universal_newlines=True,
                                      startupinfo=info,) as proc:

                    for line in proc.stdout:
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_YDL_EXECUTABLE_EVT",
                                     output=line,
                                     duration=100,
                                     status=0,
                                     )
                        if self.stop_work_thread:  # break second 'for' loop
                            proc.terminate()
                            break

                    if proc.wait():  # error
                        if 'line' not in locals():
                            line = YtdlExecDL.LINE_MSG
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_YDL_EXECUTABLE_EVT",
                                     output=line,
                                     duration=100,
                                     status=proc.wait(),
                                     )
                        logwrite('',
                                 "Exit status: %s" % proc.wait(),
                                 self.logname,
                                 YtdlExecDL.LOGDIR,
                                 )  # append exit error number
                    else:  # ok
                        wx.CallAfter(pub.sendMessage,
                                     "COUNT_EVT",
                                     count='',
                                     fsource='',
                                     destination='',
                                     duration='',
                                     end='ok'
                                     )
            except (OSError, FileNotFoundError) as err:
                excepterr = "%s" % err
                wx.CallAfter(pub.sendMessage,
                             "COUNT_EVT",
                             count=excepterr,
                             fsource='',
                             destination='',
                             duration=0,
                             end='error',
                             )
                break

            if self.stop_work_thread:  # break first 'for' loop
                proc.terminate()
                break

        time.sleep(.5)
        wx.CallAfter(pub.sendMessage, "END_EVT")
    # --------------------------------------------------------------------#

    def stop(self):
        """
        Sets the stop work thread to terminate the process
        """
        self.stop_work_thread = True
# ------------------------------------------------------------------------#


class YtdlExecEI(Thread):
    """
    YtdlExecEI it is a separate thread to run youtube-dl executable
    with subprocess class to get 'Format code' data and exit status
    from stdout/stderr output .

    """
    get = wx.GetApp()  # get videomass wx.App attribute
    EXECYDL = get.appset['EXECYDL']
    APPTYPE = get.appset['app']

    if not platform.system() == 'Windows':
        LINE_MSG = _('Unrecognized error')
    else:
        if os.path.isfile(EXECYDL):
            LINE_MSG = (_('\nRequires MSVCR100.dll\nTo resolve this problem '
                          'install: Microsoft Visual C++ 2010 Redistributable '
                          'Package (x86)'))
        else:
            LINE_MSG = _('Unrecognized error')
    # ---------------------------------------------------------------#

    def __init__(self, url):
        """
        self.url:          urls list
        """
        self.url = url
        self.status = None
        self.data = None

        if (platform.system() == 'Windows' or
                YtdlExecEI.APPTYPE == 'appimage'):
            self.ssl = '--no-check-certificate'
        else:
            self.ssl = ''

        Thread.__init__(self)
        self.start()  # start the thread (va in self.run())

    def run(self):
        """
        Subprocess initialize thread.

        """
        cmd = ('"{0}" {1} --newline --ignore-errors --ignore-config '
               '--restrict-filenames -F "{2}"'.format(YtdlExecEI.EXECYDL,
                                                      self.ssl,
                                                      self.url))
        if not platform.system() == 'Windows':
            cmd = shlex.split(cmd)
            info = None
        else:
            # Hide subprocess window on MS Windows
            info = subprocess.STARTUPINFO()
            info.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        try:
            with subprocess.Popen(cmd,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.STDOUT,
                                  universal_newlines=True,
                                  startupinfo=info,
                                  ) as proc:

                out = proc.communicate()

                if proc.returncode:  # if returncode == 1
                    if (not out[0] and not out[1] and
                            platform.system() == 'Windows'):
                        self.status = YtdlExecEI.LINE_MSG, 'error'
                    else:
                        self.status = (out[0], 'error')
                else:
                    self.status = (out[0], out[1])

        except OSError as oserr:  # executable do not exist
            self.status = ('%s' % oserr, 'error')

        self.data = self.status

        wx.CallAfter(pub.sendMessage,
                     "RESULT_EVT",
                     status=''
                     )



# -*- coding: UTF-8 -*-
"""
Name: epilogue.py
Porpose: show dialog box before start process
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: May.15.2020
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""

import wx


class Formula(wx.Dialog):
    """
   Show a dialog box before run process. It accept a couple
   of tuples: settingss and parameters.

   Example:
            settings = ("\nEXAMPLES:\n\\nExample 1:\nExample 2:\n etc."
            param = ("type 1\ntype 2\ntype 3\n etc."
    """
    # light
    LAVENDER = '#e6e6faff'
    LIGHT_SLATE = '#778899ff'
    # dark
    DARK_SLATE = '#1c2027ff'
    DARK_GREEN = '#008000'
    LIGHT_GREY = '#959595'
    # breeze-blues
    SOLARIZED = '#11303eff'

    def __init__(self, parent, settings, param, title):

        get = wx.GetApp()  # get data from bootstrap

        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)

        panel = wx.Panel(self, wx.ID_ANY, style=wx.TAB_TRAVERSAL)
        label1 = wx.StaticText(panel, wx.ID_ANY, settings)
        label2 = wx.StaticText(panel, wx.ID_ANY, param)
        self.button_1 = wx.Button(self, wx.ID_CANCEL, "")
        self.button_2 = wx.Button(self, wx.ID_OK, "")

        # ----------------------Properties----------------------#
        self.SetTitle(title)

        if get.appset['icontheme'] in ('Breeze-Blues', 'Videomass-Colours'):
            panel.SetBackgroundColour(Formula.SOLARIZED)
            label2.SetForegroundColour(Formula.LIGHT_GREY)

        elif get.appset['icontheme'] in ('Breeze-Blues',
                                         'Breeze-Dark',
                                         'Videomass-Dark'):
            panel.SetBackgroundColour(Formula.DARK_SLATE)
            label2.SetForegroundColour(Formula.LIGHT_GREY)
        else:
            panel.SetBackgroundColour(Formula.LAVENDER)
            label2.SetForegroundColour(Formula.LIGHT_SLATE)

        label1.SetForegroundColour(Formula.DARK_GREEN)
        # ---------------------- Layout ----------------------#
        bs1 = wx.BoxSizer(wx.VERTICAL)
        gr_s1 = wx.FlexGridSizer(1, 2, 0, 0)
        gr_s1.Add(label1, 0, wx.ALL | wx.ALIGN_CENTRE_VERTICAL, 5)
        gr_s1.Add(label2, 0, wx.ALL | wx.ALIGN_CENTRE_VERTICAL, 5)
        btngrid = wx.FlexGridSizer(1, 2, 0, 0)
        btngrid.Add(self.button_1, 0, wx.ALL, 5)
        btngrid.Add(self.button_2, 0, wx.ALL, 5)
        panel.SetSizer(gr_s1)
        bs1.Add(panel, 1, wx.ALL | wx.EXPAND, 5)
        bs1.Add(btngrid, flag=wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=0)
        self.SetSizer(bs1)
        bs1.Fit(self)
        self.Layout()
        # ----------------------Binders (EVT)--------------------#
        self.Bind(wx.EVT_BUTTON, self.on_cancel, self.button_1)
        self.Bind(wx.EVT_BUTTON, self.on_ok, self.button_2)

        # --------------  Event handler (callback)  --------------#

    def on_cancel(self, event):
        """
        exit from formula dialog
        """
        # self.Destroy()
        event.Skip()

    def on_ok(self, event):
        """
        get confirmation to proceed
        """
        # self.Destroy()
        event.Skip()



# -*- coding: UTF-8 -*-
"""
Name: video_stabilization.py
Porpose: FFmpeg long processing vidstab
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: May.09.2021
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
from threading import Thread
import time
import itertools
import subprocess
import platform
import wx
from pubsub import pub
if not platform.system() == 'Windows':
    import shlex


def logwrite(cmd, sterr, logname, logdir):
    """
    writes ffmpeg commands and status error during threads below
    """
    if sterr:
        apnd = "...%s\n\n" % (sterr)
    else:
        apnd = "%s\n\n" % (cmd)

    with open(os.path.join(logdir, logname), "a", encoding='utf8') as log:
        log.write(apnd)


# ------------------------------ THREADS -------------------------------#
"""
NOTE MS Windows:

subprocess.STARTUPINFO()

https://stackoverflow.com/questions/1813872/running-
a-process-in-pythonw-with-popen-without-a-console?lq=1>

NOTE capturing output in real-time (Windows, Unix):

https://stackoverflow.com/questions/1388753/how-to-get-output-
from-subprocess-popen-proc-stdout-readline-blocks-no-dat?rq=1
"""


class VidStab(Thread):
    """
    This class represents a separate thread which need
    to read the stdout/stderr in real time mode for
    different tasks.

    """
    get = wx.GetApp()  # get videomass wx.App attribute
    appdata = get.appset
    OS = appdata['ostype']
    SUFFIX = '' if appdata['filesuffix'] == 'none' else appdata['filesuffix']
    NOT_EXIST_MSG = _("Is 'ffmpeg' installed on your system?")

    def __init__(self, varargs, duration, logname, timeseq):
        """
        The 'volume' attribute may have an empty value, but it will
        have no influence on the type of conversion.
        """
        self.stop_work_thread = False  # process terminate
        self.filelist = varargs[1]  # list of files (elements)
        self.passlist = varargs[5]  # comand list set for double-pass
        self.makeduo = varargs[4]  # one more process for the duo file
        self.outputdir = varargs[3]  # output path
        self.extoutput = varargs[2]  # format (extension)
        self.duration = duration  # duration list
        self.time_seq = timeseq  # a time segment
        self.volume = varargs[7]  # volume compensation data
        self.count = 0  # count first for loop
        self.countmax = len(varargs[1])  # length file list
        self.logname = logname  # title name of file log
        self.nul = 'NUL' if VidStab.OS == 'Windows' else '/dev/null'

        spl = varargs[6].split('-vf ')[1]
        addspl = ','.join([x for x in spl.split(',') if '-vf' not
                           in x and 'vidstabtransform' not in x and
                           'unsharp' not in x])  # if other filters
        self.addflt = '' if addspl == '' else '%s,' % (addspl)

        Thread.__init__(self)
        self.start()  # start the thread (va in self.run())

    def run(self):
        """
        Subprocess initialize thread.
        """
        for (files,
             folders,
             volume,
             duration) in itertools.zip_longest(self.filelist,
                                                self.outputdir,
                                                self.volume,
                                                self.duration,
                                                fillvalue='',
                                                ):
            basename = os.path.basename(files)  # nome file senza path
            filename = os.path.splitext(basename)[0]  # nome senza ext
            source_ext = os.path.splitext(basename)[1].split('.')[1]  # ext
            outext = source_ext if not self.extoutput else self.extoutput
            outputfile = os.path.join(folders, '%s%s.%s' % (filename,
                                                            VidStab.SUFFIX,
                                                            outext))

            # --------------- first pass
            pass1 = ('"%s" %s %s -i "%s" %s %s '
                     '-y %s' % (VidStab.appdata['ffmpeg_bin'],
                                VidStab.appdata['ffmpegloglev'],
                                self.time_seq,
                                files,
                                self.passlist[0],
                                VidStab.appdata['ffthreads'],
                                self.nul,
                                ))
            self.count += 1
            count = ('File %s/%s - Pass One\n'
                     'Video stabilization detect...' % (self.count,
                                                        self.countmax)
                     )
            cmd = ('%s\nSource: "%s"\nDestination: "%s"\n\n'
                   '[COMMAND]:\n%s' % (count, files, self.nul, pass1))

            wx.CallAfter(pub.sendMessage,
                         "COUNT_EVT",
                         count=count,
                         fsource='Source:  "%s"' % files,
                         destination='Destination:  "%s"' % self.nul,
                         duration=duration,
                         end='',
                         )
            logwrite(cmd,
                     '',
                     self.logname,
                     VidStab.appdata['logdir'],
                     )  # write n/n + command only

            if not VidStab.OS == 'Windows':
                pass1 = shlex.split(pass1)
                info = None
            else:  # Hide subprocess window on MS Windows
                info = subprocess.STARTUPINFO()
                info.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            try:
                with subprocess.Popen(pass1,
                                      stderr=subprocess.PIPE,
                                      bufsize=1,
                                      universal_newlines=True,
                                      startupinfo=info,) as proc1:

                    for line in proc1.stderr:
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output=line,
                                     duration=duration,
                                     status=0
                                     )
                        if self.stop_work_thread:  # break second 'for' loop
                            proc1.terminate()
                            break

                    if proc1.wait():  # will add '..failed' to txtctrl
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output=line,
                                     duration=duration,
                                     status=proc1.wait(),
                                     )
                        logwrite('',
                                 "Exit status: %s" % proc1.wait(),
                                 self.logname,
                                 VidStab.appdata['logdir']
                                 )  # append exit error number

            except (OSError, FileNotFoundError) as err:
                excepterr = "%s\n  %s" % (err, VidStab.NOT_EXIST_MSG)
                wx.CallAfter(pub.sendMessage,
                             "COUNT_EVT",
                             count=excepterr,
                             fsource='',
                             destination='',
                             duration=0,
                             end='error',
                             )
                break

            if self.stop_work_thread:  # break first 'for' loop
                proc1.terminate()
                break  # fermo il ciclo for, altrimenti passa avanti

            if proc1.wait() == 0:  # will add '..terminated' to txtctrl
                wx.CallAfter(pub.sendMessage,
                             "COUNT_EVT",
                             count='',
                             fsource='',
                             destination='',
                             duration='',
                             end='ok'
                             )
            # --------------- second pass ----------------#
            pass2 = ('"%s" %s %s -i "%s" %s %s %s '
                     '-y "%s/%s%s.%s"' % (VidStab.appdata['ffmpeg_bin'],
                                          VidStab.appdata['ffmpegloglev'],
                                          self.time_seq,
                                          files,
                                          self.passlist[1],
                                          volume,
                                          VidStab.appdata['ffthreads'],
                                          folders,
                                          filename,
                                          VidStab.SUFFIX,
                                          outext,
                                          ))
            count = ('File %s/%s - Pass Two\nVideo transform...' % (
                     self.count, self.countmax,))

            cmd = ('%s\nSource: "%s"\nDestination: "%s"\n\n'
                   '[COMMAND]:\n%s' % (count, files, outputfile, pass2))

            wx.CallAfter(pub.sendMessage,
                         "COUNT_EVT",
                         count=count,
                         fsource='Source:  "%s"' % files,
                         destination='Destination:  "%s"' % outputfile,
                         duration=duration,
                         end='',
                         )
            logwrite(cmd, '', self.logname, VidStab.appdata['logdir'])

            if not VidStab.OS == 'Windows':
                pass2 = shlex.split(pass2)
                info = None
            else:  # Hide subprocess window on MS Windows
                info = subprocess.STARTUPINFO()
                info.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            with subprocess.Popen(pass2,
                                  stderr=subprocess.PIPE,
                                  bufsize=1,
                                  universal_newlines=True,
                                  startupinfo=info,) as proc2:

                for line2 in proc2.stderr:
                    wx.CallAfter(pub.sendMessage,
                                 "UPDATE_EVT",
                                 output=line2,
                                 duration=duration,
                                 status=0,
                                 )
                    if self.stop_work_thread:
                        proc2.terminate()
                        break

                if proc2.wait():  # will add '..failed' to txtctrl
                    wx.CallAfter(pub.sendMessage,
                                 "UPDATE_EVT",
                                 output=line,
                                 duration=duration,
                                 status=proc2.wait(),
                                 )
                    logwrite('',
                             "Exit status: %s" % proc2.wait(),
                             self.logname,
                             VidStab.appdata['logdir'],
                             )  # append exit status error

            if self.stop_work_thread:  # break first 'for' loop
                proc2.terminate()
                break  # fermo il ciclo for, altrimenti passa avanti

            if proc2.wait() == 0:  # will add '..terminated' to txtctrl
                wx.CallAfter(pub.sendMessage,
                             "COUNT_EVT",
                             count='',
                             fsource='',
                             destination='',
                             duration='',
                             end='ok'
                             )

            # --------------- make duo ----------------#
            if self.makeduo:
                outduo = os.path.join(folders, '%s%s_DUO.%s' % (filename,
                                                                VidStab.SUFFIX,
                                                                outext))
                pass3 = ('"%s" %s %s -i "%s" %s -vf "[in] %spad=2*iw:ih '
                         '[left]; movie=%s [right]; '
                         '[left][right] overlay=main_w/2:0 [out]" -y '
                         '"%s"' % (VidStab.appdata['ffmpeg_bin'],
                                   VidStab.appdata['ffmpegloglev'],
                                   self.time_seq,
                                   files,
                                   VidStab.appdata['ffthreads'],
                                   self.addflt,
                                   outputfile,
                                   outduo,
                                   ))
                count = 'File %s/%s\nMake duo...' % (self.count, self.countmax)
                cmd = ('%s\nSource: "%s"\nDestination: "%s"\n\n'
                       '[COMMAND]:\n%s' % (count, files, outputfile, pass3))

                wx.CallAfter(pub.sendMessage,
                             "COUNT_EVT",
                             count=count,
                             fsource='Source:  "%s"' % files,
                             destination='Destination: "%s"' % outduo,
                             duration=duration,
                             end='',
                             )
                logwrite(cmd, '', self.logname, VidStab.appdata['logdir'])

                if not VidStab.OS == 'Windows':
                    pass3 = shlex.split(pass3)
                    info = None
                else:  # Hide subprocess window on MS Windows
                    info = subprocess.STARTUPINFO()
                    info.dwFlags |= subprocess.STARTF_USESHOWWINDOW
                with subprocess.Popen(pass3,
                                      stderr=subprocess.PIPE,
                                      bufsize=1,
                                      universal_newlines=True,
                                      startupinfo=info,) as proc3:

                    for line3 in proc3.stderr:
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output=line3,
                                     duration=duration,
                                     status=0,
                                     )
                        if self.stop_work_thread:
                            proc3.terminate()
                            break

                    if proc3.wait():  # will add '..failed' to txtctrl
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output=line,
                                     duration=duration,
                                     status=proc3.wait(),
                                     )
                        logwrite('',
                                 "Exit status: %s" % proc3.wait(),
                                 self.logname,
                                 VidStab.appdata['logdir'],
                                 )  # append exit error number

                if self.stop_work_thread:  # break first 'for' loop
                    proc3.terminate()
                    break  # fermo il ciclo for, altrimenti passa avanti

                if proc3.wait() == 0:  # will add '..terminated' to txtctrl
                    wx.CallAfter(pub.sendMessage,
                                 "COUNT_EVT",
                                 count='',
                                 fsource='',
                                 destination='',
                                 duration='',
                                 end='ok'
                                 )
        time.sleep(.5)
        wx.CallAfter(pub.sendMessage, "END_EVT")
    # --------------------------------------------------------------------#

    def stop(self):
        """
        Sets the stop work thread to terminate the process
        """
        self.stop_work_thread = True



# -*- coding: UTF-8 -*-
"""
Name: ffprobe_parser.py
Porpose: cross-platform parsing class for ffprobe
Compatibility: Python3, Python2
Platform: all platforms
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: May.11.2021
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101
########################################################

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import subprocess
import platform
import re
if not platform.system() == 'Windows':
    import shlex


class FFProbe():
    """
    FFProbe wraps the ffprobe command and pulls the data into
    an object form. The `parse` argument defines the parser's behavior.

    If you specify `parser=True' (default) which give an auto-parsed output
    in a object list for each instance (see examples below). The `select`,
    `entries`, and `writer` arguments will be ignored;

    If you specify `parse=False` you  get a customized output characterized
    by the arguments you define. Note that in this case the output given by
    custom_output() instance will always be given in a string object.
    For instance, if you use `writer=json' you must use `eval' function
    to obtain a dict object.

    ---------------------
    USE with `parse=True` (default):
    ---------------------

    >>> from ffprobe_parser import FFProbe

    >>> data = FFProbe('ffprobe_url',
                       'filename_url',
                       parse=True,
                       pretty=True,
                       select=None,
                       entries=None,
                       show_format=True,
                       show_streams=True,
                       writer=None
                       )

        First handle possible exceptions, example:

            if data.error_check():
                print ("Some Error:  %s" % (data.error))

            else:
                video = data.video_stream()
                audio = data.audio_stream()
                subtitle = data.subtitle_stream()
                dataformat = data.data_format())

                formatdict = dict()
                for items in dataformat:
                    for line in items:
                        if '=' in line:
                            k, v = line.split('=')
                            formatdict[k.strip()] = v.strip()
                print(formatdict)

    ----------------------
    USE with `parse=False`:
    ----------------------

    >>> from ffprobe_parser import FFProbe

        Get simple output data:
        -----------------------

            `data = FFProbe(FFPROBE_URL,
                            filename_url,
                            parse=False,
                            writer='xml')
                            )`

            First handle possible exceptions, example:

                if data.error_check():
                    print("Some Error:  %s" % (data.error))

                else:
                    print(data.custom_output())

        To get a kind of output:
        ------------------------

             A example entry of a first audio streams section

            `data = FFProbe(FFPROBE_URL,
                            filename_url,
                            parse=False,
                            pretty=True,
                            select='a:0',
                            entries='stream=codec_type',
                            show_format=False,
                            show_streams=False,
                            writer='compact=nk=1:p=0'
                            )`

            First handle possible exceptions, example:

                if data.error_check():
                    print("Some Error:  %s" % (data.error))
                else:
                    print(data.custom_output().strip())

            The `entries` arg is the key to search some entry on sections

                entries='stream=codec_type,codec_name,bits_per_sample'
                entries='format=duration'

            The `select` arg is the key to select a specified section

                select='' # select all sections
                select='v' # select all video sections
                select='v:0' # select first video section

            The `writer` arg alias:

                writer='default=nw=1:nk=1'
                writer='default=noprint_wrappers=1:nokey=1'

                available writers name are:

                `default`, `compact`, `csv`, `flat`, `ini`, `json` and `xml`

                Options are list of key=value pairs, separated by ":"

                See `man ffprobe`

    ------------------------------------------------
    [i] This class was partially inspired to:
    ------------------------------------------------
        <https://github.com/simonh10/ffprobe/blob/master/ffprobe/ffprobe.py>

    """

    def __init__(self, FFPROBE_URL, filename, parse=True,
                 pretty=True, select=None, entries=None,
                 show_format=True, show_streams=True, writer=None):
        """
        -------------------
        Parameters meaning:
        -------------------
            FFPROBE_URL     command name by $PATH defined or a binary url
            filename_url    a pathname appropriately quoted
            parse           defines the output mode
            show_format     show format informations
            show_streams    show all streams information
            select          select which section to show (''= all sections)
            entries         get one or more entries
            pretty          get human values or machine values
            writer          define a format of printing output

        --------------------------------------------------
        [?] to know the meaning of the above options, see:
        --------------------------------------------------
            <http://trac.ffmpeg.org/wiki/FFprobeTips>
            <https://slhck.info/ffmpeg-encoding-course/#/46>

        -------------------------------------------------------------------
        The ffprobe command has stdout and stderr (unlike ffmpeg and ffplay)
        which allows me to initialize separate attributes also for errors
        """
        self.error = False
        self.mediastreams = []
        self.mediaformat = []
        self.video = []
        self.audio = []
        self._format = []
        self.subtitle = []
        self.writer = None
        self.datalines = []
        pretty = '-pretty' if pretty is True else 'no_pretty'
        show_format = '-show_format' if show_format is True else ''
        show_streams = '-show_streams' if show_streams is True else ''
        select = '-select_streams %s' % select if select else ''
        entries = '-show_entries %s' % entries if entries else ''
        writer = '-of %s' % writer if writer else '-of default'

        if parse:
            cmnd = ('"%s" -i "%s" -v error %s %s %s '
                    '-print_format default' % (FFPROBE_URL,
                                               filename,
                                               pretty,
                                               show_format,
                                               show_streams,)
                    )
        else:
            cmnd = '"%s" -i "%s" -v error %s %s %s %s %s %s' % (FFPROBE_URL,
                                                                filename,
                                                                pretty,
                                                                select,
                                                                entries,
                                                                show_format,
                                                                show_streams,
                                                                writer
                                                                )
        if not platform.system() == 'Windows':
            cmnd = shlex.split(cmnd)
            info = None
        else:
            # Hide subprocess window on MS Windows
            info = subprocess.STARTUPINFO()
            info.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        try:
            with subprocess.Popen(cmnd,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
                                  universal_newlines=True,
                                  startupinfo=info,
                                  ) as proc:
                output, error = proc.communicate()
                if proc.returncode:
                    self.error = error
                if parse:
                    self.parser(output)
                else:
                    self.writer = output

        except (OSError, FileNotFoundError) as err:
            self.error = err
            return
    # -------------------------------------------------------------#

    def parser(self, output):
        """
        Indexes the catalogs [STREAM\\] and [FORMAT\\] given by
        the default output of FFprobe
        """
        probing = output.split('\n')  # create list with strings element

        for streams in probing:
            if re.match('\\[STREAM\\]', streams):
                self.datalines = []

            elif re.match('\\[\\/STREAM\\]', streams):
                self.mediastreams.append(self.datalines)
                self.datalines = []
            else:
                self.datalines.append(streams)

        for fformat in probing:
            if re.match('\\[FORMAT\\]', fformat):
                self.datalines = []

            elif re.match('\\[\\/FORMAT\\]', fformat):
                self.mediaformat.append(self.datalines)
                self.datalines = []
            else:
                self.datalines.append(fformat)

    # --------------------------------------------------------------#

    def video_stream(self):
        """
        Return a metadata list for video stream. If there is not
        data video return a empty list
        """
        for datastream in self.mediastreams:
            if 'codec_type=video' in datastream:
                self.video.append(datastream)
        return self.video
    # --------------------------------------------------------------#

    def audio_stream(self):
        """
        Return a metadata list for audio stream. If there is not
        data audio return a empty list
        """
        for datastream in self.mediastreams:
            if 'codec_type=audio' in datastream:
                self.audio.append(datastream)
        return self.audio
    # --------------------------------------------------------------#

    def subtitle_stream(self):
        """
        Return a metadata list for subtitle stream. If there is not
        data subtitle return a empty list
        """
        for datastream in self.mediastreams:
            if 'codec_type=subtitle' in datastream:
                self.subtitle.append(datastream)
        return self.subtitle
    # --------------------------------------------------------------#

    def data_format(self):
        """
        Return a metadata list for data format. If there is not
        data format return a empty list
        """
        for dataformat in self.mediaformat:
            self._format.append(dataformat)
        return self._format
    # --------------------------------------------------------------#

    def custom_output(self):
        """
        Print output defined by writer argument. To use this feature
        you must specify parse=False, example:

        `data = FFProbe(filename_url,
                        FFPROBE_URL,
                        parse=False,
                        writer='json')`

        Then, to get output data call this method:

        output = data.custom_output()

        Valid writers are: `default`, `json`, `compact`, `csv`, `flat`,
        `ini` and `xml` .
        """
        return self.writer
    # ----------------------------------------------------------------#

    def error_check(self):
        """
        check for errors on stderr of the ffprobe command. It also
        handles the IOError exception. You can use this interface
        before using all other methods of this class.

        """
        return self.error if self.error else None



# -*- coding: UTF-8 -*-
"""
Name: configurator.py
Porpose: Set Videomass configuration and appearance on startup
Compatibility: Python3
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Sep.04.2021
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101

 This file is part of Videomass.

    Videomass is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Videomass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import sys
import shutil
import platform
from videomass3.vdms_utils.utils import copydir_recursively
from videomass3.vdms_utils.utils import copy_restore


def parsing_fileconf(fileconf):
    """
    Make parsing of the configuration file and return
    object list with the current program settings data.
    """
    with open(fileconf, 'r', encoding='utf8') as fget:
        fconf = fget.readlines()
    lst = [line.strip() for line in fconf if not line.startswith('#')]
    dataconf = [x for x in lst if x]  # list without empties values

    return None if not dataconf else dataconf


def get_pyinstaller():
    """
    Get pyinstaller-based package attributes to determine
    how to use sys.executable
    When a bundled app starts up, the bootloader sets the sys.frozen
    attribute and stores the absolute path to the bundle folder
    in sys._MEIPASS.
    For a one-folder bundle, this is the path to that folder.
    For a one-file bundle, this is the path to the temporary folder
    created by the bootloader.
    """
    if getattr(sys, 'frozen', False) and hasattr(sys, '_MEIPASS'):
        print('getattr frozen, hasattr _MEIPASS')
        frozen, meipass = True, True
        mpath = getattr(sys, '_MEIPASS', os.path.abspath(__file__))
        data_locat = mpath

    else:
        frozen, meipass = False, False
        mpath = os.path.realpath(os.path.abspath(__file__))
        data_locat = os.path.dirname(os.path.dirname(mpath))

    return frozen, meipass, mpath, data_locat


def conventional_paths():
    """
    Establish the conventional paths based on OS by installation

    """
    user_name = os.path.expanduser('~')

    if platform.system() == 'Windows':
        dirpath = "\\AppData\\Roaming\\videomass\\videomass.conf"
        file_conf = os.path.join(user_name + dirpath)
        dir_conf = os.path.join(user_name + "\\AppData\\Roaming\\videomass")
        log_dir = os.path.join(dir_conf, 'log')  # logs
        cache_dir = os.path.join(dir_conf, 'cache')  # updates executable

    elif platform.system() == "Darwin":
        dirpath = "Library/Application Support/videomass/videomass.conf"
        file_conf = os.path.join(user_name, dirpath)
        dir_conf = os.path.join(user_name, os.path.dirname(dirpath))
        log_dir = os.path.join(user_name, "Library/Logs/videomass")
        cache_dir = os.path.join(user_name, "Library/Caches/videomass")

    else:  # Linux, FreeBsd, etc.
        dirpath = ".config/videomass/videomass.conf"
        file_conf = os.path.join(user_name, dirpath)
        dir_conf = os.path.join(user_name, ".config/videomass")
        log_dir = os.path.join(user_name, ".local/share/videomass/log")
        cache_dir = os.path.join(user_name, ".cache/videomass")

    return file_conf, dir_conf, log_dir, cache_dir


def portable_paths(portdir):
    """
    Make portable-data paths based on OS by a portable mode

    """
    if platform.system() == 'Windows':
        file_conf = portdir + "\\portable_data\\videomass.conf"
        dir_conf = portdir + "\\portable_data"
        log_dir = os.path.join(dir_conf, 'log')  # logs
        cache_dir = os.path.join(dir_conf, 'cache')  # updates executable
    else:
        file_conf = portdir + "/portable_data/videomass.conf"
        dir_conf = portdir + "/portable_data"
        log_dir = os.path.join(dir_conf, 'log')  # logs
        cache_dir = os.path.join(dir_conf, 'cache')  # updates executable

    return file_conf, dir_conf, log_dir, cache_dir


class DataSource():
    """
    DataSource class determines the Videomass's configuration
    according to the used Operating System (Linux/*BSD, MacOS, Windows).
    Remember that all specifications defined in this class refer
    only to the following packaging methods:

        - Videomass Python package by PyPi (cross-platform, multiuser/user)
        - Linux's distributions packaging (multiuser)
        - Bundled app by pyinstaller (windowed for MacOs and Windows)
        - AppImage for Linux (user)

        * multiuser: root installation
        * user: local installation

    """
    # NOTE: For develop. test, add this line to use portable data during
          # a local source code execution (NOT OTHER):
        # elif os.path.isdir(os.path.join(DATA_LOCAT,'portable_data')):
            # (FILE_CONF,
             # DIR_CONF,
             # LOG_DIR,
             # CACHE_DIR) = portable_paths(DATA_LOCAT)

    FROZEN, MEIPASS, MPATH, DATA_LOCAT = get_pyinstaller()
    portdirname = os.path.dirname(sys.executable)
    portdir = os.path.join(portdirname, 'portable_data')

    if FROZEN and MEIPASS and os.path.isdir(portdir):
        # if portdir is true, make application really portable
        FILE_CONF, DIR_CONF, LOG_DIR, CACHE_DIR = portable_paths(portdirname)

    else:
        #
        FILE_CONF, DIR_CONF, LOG_DIR, CACHE_DIR = conventional_paths()
    # -------------------------------------------------------------------

    def __init__(self):
        """
        Given the pathnames defined by `DATA_LOCAT` it performs
        the initialization described in DataSource.

            `self.srcpath` > configuration folder for recovery
            `self.icodir` > set of icons
            `self.localepath` > locale folder

        """
        self.apptype = None  # appimage, pyinstaller on None
        self.workdir = os.path.dirname(os.path.dirname
                                       (os.path.dirname(DataSource.MPATH))
                                       )
        self.localepath = os.path.join(DataSource.DATA_LOCAT, 'locale')
        self.srcpath = os.path.join(DataSource.DATA_LOCAT, 'share')
        self.icodir = os.path.join(DataSource.DATA_LOCAT, 'art', 'icons')
        self.ffmpeg_pkg = os.path.join(DataSource.DATA_LOCAT, 'FFMPEG')
        launcher = os.path.isfile('%s/launcher' % self.workdir)

        if DataSource.FROZEN and DataSource.MEIPASS or launcher:
            print('frozen=%s meipass=%s launcher=%s' % (DataSource.FROZEN,
                                                        DataSource.MEIPASS,
                                                        launcher
                                                        ))
            self.apptype = 'pyinstaller' if launcher is False else None
            self.videomass_icon = "%s/videomass.png" % self.icodir

        else:
            if platform.system() == 'Windows':  # Installed with pip command
                print('Win32 executable=%s' % sys.executable)
                # HACK check this
                # dirname = os.path.dirname(sys.executable)
                # pythonpath = os.path.join(dirname, 'Script', 'videomass')
                # self.icodir = dirname + '\\share\\videomass\\icons'
                self.videomass_icon = self.icodir + "\\videomass.png"

            elif ('/tmp/.mount_' in sys.executable or os.path.exists(
                  os.path.dirname(os.path.dirname(os.path.dirname(
                      sys.argv[0]))) + '/AppRun')):
                # embedded on python appimage
                print('Embedded on python appimage')
                self.apptype = 'appimage'
                userbase = os.path.dirname(os.path.dirname(sys.argv[0]))
                pixmaps = '/share/pixmaps/videomass.png'
                self.videomass_icon = os.path.join(userbase + pixmaps)

            else:
                binarypath = shutil.which('videomass')

                if binarypath == '/usr/local/bin/videomass':
                    print('executable=%s' % binarypath)
                    # pip as super user, usually Linux, MacOs, Unix
                    share = '/usr/local/share/pixmaps'
                    self.videomass_icon = share + '/videomass.png'

                elif binarypath == '/usr/bin/videomass':
                    print('executable=%s' % binarypath)
                    # installed via apt, rpm, etc, usually Linux
                    share = '/usr/share/pixmaps'
                    self.videomass_icon = share + "/videomass.png"

                else:
                    print('executable=%s' % binarypath)
                    # pip as normal user, usually Linux, MacOs, Unix
                    userbase = os.path.dirname(os.path.dirname(binarypath))
                    pixmaps = '/share/pixmaps/videomass.png'
                    self.videomass_icon = os.path.join(userbase + pixmaps)
    # ---------------------------------------------------------------------

    def get_fileconf(self):
        """
        Get videomass configuration data by videomass.conf file.
        Returns a dict object.

        This method performs the following main steps:

            1) Checks user videomass configuration dir; if not exists try to
               restore from self.srcpath
            2) Read the videomass.conf file; if not exists try to restore
               from self.srcpath
            3) Checks presets folder; if not exists try to restore from
               self.srcpath

        Note that when `copyerr` is not False, it raise a fatal error on
        videomass bootstrap.
        """
        copyerr = False
        existfileconf = True  # True > found, False > not found

        if os.path.exists(DataSource.DIR_CONF):  # if ~/.conf/videomass dir
            if os.path.isfile(DataSource.FILE_CONF):
                userconf = parsing_fileconf(DataSource.FILE_CONF)
                if not userconf:
                    existfileconf = False
                if float(userconf[0]) != 3.1:
                    existfileconf = False
            else:
                existfileconf = False

            if not existfileconf:  # try to restore only videomass.conf
                fcopy = copy_restore('%s/videomass.conf' % self.srcpath,
                                     DataSource.FILE_CONF)
                if fcopy:
                    copyerr, userconf = fcopy, None
                else:
                    userconf = parsing_fileconf(DataSource.FILE_CONF)
                    # read again file conf

            if not os.path.exists(os.path.join(DataSource.DIR_CONF,
                                               "presets")):
                # try to restoring presets directory on videomass dir
                drest = copydir_recursively(os.path.join(self.srcpath,
                                                         "presets"),
                                            DataSource.DIR_CONF)
                if drest:
                    copyerr, userconf = drest, None

        else:  # try to restore entire configuration directory
            dconf = copydir_recursively(self.srcpath,
                                        os.path.dirname(DataSource.DIR_CONF),
                                        "videomass")
            userconf = parsing_fileconf(DataSource.FILE_CONF)
            # read again file conf
            if dconf:
                copyerr, userconf = dconf, None

        return ({'ostype': platform.system(),
                 'srcpath': self.srcpath,
                 'fatalerr': copyerr,
                 'localepath': self.localepath,
                 'fileconfpath': DataSource.FILE_CONF,
                 'workdir': self.workdir,
                 'confdir': DataSource.DIR_CONF,
                 'logdir': DataSource.LOG_DIR,
                 'cachedir': DataSource.CACHE_DIR,
                 'FFMPEG_videomass_pkg': self.ffmpeg_pkg,
                 'app': self.apptype,
                 'confversion': userconf[0],
                 'outputfile': userconf[1],
                 'ffthreads': userconf[2],
                 'ffplayloglev': userconf[3],
                 'ffmpegloglev': userconf[4],
                 'ffmpeg_local': userconf[5],
                 'ffmpeg_bin': userconf[6],
                 'ffprobe_local': userconf[7],
                 'ffprobe_bin': userconf[8],
                 'ffplay_local': userconf[9],
                 'ffplay_bin': userconf[10],
                 'icontheme': userconf[11],
                 'toolbarsize': userconf[12],
                 'toolbarpos': userconf[13],
                 'toolbartext': userconf[14],
                 'clearcache': userconf[15],
                 'enable_youtubedl': userconf[16],
                 'outputfile_samedir': userconf[17],
                 'filesuffix': userconf[18],
                 'outputdownload': userconf[19],
                 'playlistsubfolder': userconf[20]}
                )
    # --------------------------------------------------------------------

    def icons_set(self, icontheme):
        """
        Determines icons set assignment defined on the configuration
        file (see `Set icon themes map:`, on paragraph `6- GUI setup`
        by Videomass.conf).
        Returns a icontheme dict object.

        """
        keys = ('videomass', 'A/V-Conv', 'startconv', 'fileproperties',
                'playback', 'concatenate', 'preview', 'clear',
                'profile_append', 'scale', 'crop', 'rotate', 'deinterlace',
                'denoiser', 'statistics', 'settings', 'audiovolume',
                'youtube', 'presets_manager', 'profile_add', 'profile_del',
                'profile_edit', 'previous', 'next', 'startdownload',
                'download_properties', 'stabilizer', 'listindx')

        ext = 'svg' if 'wx.svg' in sys.modules else 'png'

        iconames = {'Videomass-Light':  # Videomass icons for light themes
                    {'x48': '%s/Sign_Icons/48x48_light' % self.icodir,
                     'x16': '%s/Videomass-Light/16x16' % self.icodir,
                     'x22': '%s/Videomass-Light/24x24' % self.icodir},
                    'Videomass-Dark':  # Videomass icons for dark themes
                    {'x48': '%s/Sign_Icons/48x48_dark' % self.icodir,
                     'x16': '%s/Videomass-Dark/16x16' % self.icodir,
                     'x22': '%s/Videomass-Dark/24x24' % self.icodir},
                    'Videomass-Colours':  # Videomass icons for all themes
                    {'x48': '%s/Sign_Icons/48x48' % self.icodir,
                     'x16': '%s/Videomass-Colours/16x16' % self.icodir,
                     'x22': '%s/Videomass-Colours/24x24' % self.icodir},
                    'Breeze':  # Breeze for light themes
                    {'x48': '%s/Sign_Icons/48x48_light' % self.icodir,
                     'x16': '%s/Breeze/16x16' % self.icodir,
                     'x22': '%s/Breeze/22x22' % self.icodir},
                    'Breeze-Dark':  # breeze for dark themes
                    {'x48': '%s/Sign_Icons/48x48_dark' % self.icodir,
                     'x16': '%s/Breeze-Dark/16x16' % self.icodir,
                     'x22': '%s/Breeze-Dark/22x22' % self.icodir},
                    'Breeze-Blues':  # breeze custom colorized for all themes
                    {'x48': '%s/Sign_Icons/48x48' % self.icodir,
                     'x16': '%s/Breeze-Blues/16x16' % self.icodir,
                     'x22': '%s/Breeze-Blues/22x22' % self.icodir}
                    }

        choose = iconames.get(icontheme)  # set appropriate icontheme

        iconset = (self.videomass_icon,
                   '%s/icon_videoconversions.%s' % (choose.get('x48'), ext),
                   '%s/convert.%s' % (choose.get('x22'), ext),
                   '%s/properties.%s' % (choose.get('x22'), ext),
                   '%s/playback.%s' % (choose.get('x16'), ext),
                   '%s/icon_concat.%s' % (choose.get('x48'), ext),
                   '%s/preview.%s' % (choose.get('x16'), ext),
                   '%s/edit-clear.%s' % (choose.get('x16'), ext),
                   '%s/profile-append.%s' % (choose.get('x22'), ext),
                   '%s/transform-scale.%s' % (choose.get('x16'), ext),
                   '%s/transform-crop.%s' % (choose.get('x16'), ext),
                   '%s/transform-rotate.%s' % (choose.get('x16'), ext),
                   '%s/deinterlace.%s' % (choose.get('x16'), ext),
                   '%s/denoise.%s' % (choose.get('x16'), ext),
                   '%s/statistics.%s' % (choose.get('x16'), ext),
                   '%s/configure.%s' % (choose.get('x16'), ext),
                   '%s/player-volume.%s' % (choose.get('x16'), ext),
                   '%s/icon_youtube.%s' % (choose.get('x48'), ext),
                   '%s/icon_prst_mng.%s' % (choose.get('x48'), ext),
                   '%s/newprf.%s' % (choose.get('x16'), ext),
                   '%s/delprf.%s' % (choose.get('x16'), ext),
                   '%s/editprf.%s' % (choose.get('x16'), ext),
                   '%s/go-previous.%s' % (choose.get('x22'), ext),
                   '%s/go-next.%s' % (choose.get('x22'), ext),
                   '%s/download.%s' % (choose.get('x22'), ext),
                   '%s/statistics.%s' % (choose.get('x22'), ext),
                   '%s/stabilizer.%s' % (choose.get('x16'), ext),
                   '%s/playlist-append.%s' % (choose.get('x16'), ext)
                   )
        values = [os.path.join(norm) for norm in iconset]  # normalize pathns

        return dict(zip(keys, values))



# -*- coding: UTF-8 -*-
"""
Name: opendir.py
Porpose: open file browser on given pathname
Compatibility: Python3 (Unix, Windows)
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: May.11.2021
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import subprocess
import os


def browse(opsyst, pathname):
    """
    open file browser in a specific location (OS independent)

    """
    status = 'Unrecognized error'

    if opsyst == 'Windows':
        try:
            os.startfile(os.path.realpath(pathname))

        except FileNotFoundError as pathnotfound:
            return '%s' % pathnotfound

        except Exception as anyerr:
            return '%s' % anyerr

        return None

    if opsyst == 'Darwin':
        cmd = ['open', pathname]

    else:  # xdg-open *should* be supported by recent Gnome, KDE, Xfce
        cmd = ['xdg-open', pathname]

    try:
        with subprocess.Popen(cmd,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT,
                              universal_newlines=True,  # mod text
                              ) as proc:

            out = proc.communicate()

            if proc.returncode:  # if returncode == 1
                status = out[0]
            else:
                status = None

    except (OSError, FileNotFoundError) as oserr:  # exec. do not exist
        status = '%s' % oserr

    return status

    """
    NOTE The following code work, but on MS-Windows it show a short of
         Dos-window
    -----------------

    try:
        p = subprocess.run(cmd)
        if p.stderr:
            return(p.stderr.decode())
            '''
            if not *capture_output=True* on subprocess instance
            use .decode() here.
            '''
    except FileNotFoundError as err:
        return('%s' % (err))
    """
